@@ -1,7 +1,10 @@
+//! A command which prints the current values of the realtime and monotonic
+//! clocks it's given.
+
 #[cfg(not(windows))]
 fn main() {
     println!(
-        "Raeal time: {:?}",
+        "Real time: {:?}",
         rustix::time::clock_gettime(rustix::time::ClockId::Realtime)
     );
     println!(
