@@ -323,28 +323,28 @@ pub(crate) fn eventfd(initval: u32, flags: EventfdFlags) -> io::Result<OwnedFd>
     unsafe { syscall_ret_owned_fd(c::syscall(c::SYS_eventfd2, initval, flags.bits())) }
 }
 
-#[cfg(target_os = "linux")]
+#[cfg(any(target_os = "android", target_os = "linux"))]
 #[inline]
 pub(crate) fn ioctl_blksszget(fd: BorrowedFd) -> io::Result<u32> {
     let mut result = MaybeUninit::<c::c_uint>::uninit();
     unsafe {
         ret(libc::ioctl(
             borrowed_fd(fd),
-            libc::BLKSSZGET as c::c_ulong,
+            libc::BLKSSZGET,
             result.as_mut_ptr(),
         ))?;
         Ok(result.assume_init() as u32)
     }
 }
 
-#[cfg(target_os = "linux")]
+#[cfg(any(target_os = "android", target_os = "linux"))]
 #[inline]
 pub(crate) fn ioctl_blkpbszget(fd: BorrowedFd) -> io::Result<u32> {
     let mut result = MaybeUninit::<c::c_uint>::uninit();
     unsafe {
         ret(libc::ioctl(
             borrowed_fd(fd),
-            libc::BLKPBSZGET as c::c_ulong,
+            libc::BLKPBSZGET,
             result.as_mut_ptr(),
         ))?;
         Ok(result.assume_init() as u32)
@@ -694,8 +694,15 @@ pub(crate) unsafe fn mlock_with(
     length: usize,
     flags: MlockFlags,
 ) -> io::Result<()> {
-    assert_eq!(flags.bits(), 0, "libc doesn't define `MLOCK_*` yet");
-    ret(c::mlock(addr, length))
+    weak_or_syscall! {
+        fn mlock2(
+            addr: *const c::c_void,
+            len: c::size_t,
+            flags: c::c_int
+        ) via SYS_mlock2 -> c::c_int
+    }
+
+    ret(mlock2(addr, length, flags.bits()))
 }
 
 /// # Safety
