@@ -3,12 +3,13 @@
 use super::super::c;
 use super::super::conv::{borrowed_fd, ret, ret_owned_fd, ret_send_recv, send_recv_len};
 use super::ext::{in6_addr_new, in_addr_new};
-#[cfg(not(windows))]
+#[cfg(unix)]
 use super::SocketAddrUnix;
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 use super::{
-    encode_sockaddr_v4, encode_sockaddr_v6, read_sockaddr_os, AcceptFlags, AddressFamily, Protocol,
-    RecvFlags, SendFlags, Shutdown, SocketFlags, SocketType,
+    encode_sockaddr_v4, encode_sockaddr_v6, initialize_family_to_unspec, maybe_read_sockaddr_os,
+    read_sockaddr_os, AcceptFlags, AddressFamily, Protocol, RecvFlags, SendFlags, Shutdown,
+    SocketFlags, SocketType,
 };
 use crate::as_ptr;
 use crate::fd::BorrowedFd;
@@ -50,10 +51,16 @@ pub(crate) fn recvfrom(
     fd: BorrowedFd<'_>,
     buf: &mut [u8],
     flags: RecvFlags,
-) -> io::Result<(usize, SocketAddrAny)> {
+) -> io::Result<(usize, Option<SocketAddrAny>)> {
     unsafe {
         let mut storage = MaybeUninit::<c::sockaddr_storage>::uninit();
         let mut len = size_of::<c::sockaddr_storage>() as c::socklen_t;
+
+        // `recvfrom` does not write to the storage if the socket is
+        // connection-oriented sockets, so we initialize the family field to
+        // `AF_UNSPEC` so that we can detect this case.
+        initialize_family_to_unspec(storage.as_mut_ptr());
+
         let nread = ret_send_recv(c::recvfrom(
             borrowed_fd(fd),
             buf.as_mut_ptr().cast(),
@@ -64,7 +71,7 @@ pub(crate) fn recvfrom(
         ))?;
         Ok((
             nread as usize,
-            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+            maybe_read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
         ))
     }
 }
@@ -259,7 +266,7 @@ pub(crate) fn accept_with(sockfd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Res
 }
 
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
-pub(crate) fn acceptfrom(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddrAny)> {
+pub(crate) fn acceptfrom(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, Option<SocketAddrAny>)> {
     unsafe {
         let mut storage = MaybeUninit::<c::sockaddr_storage>::uninit();
         let mut len = size_of::<c::sockaddr_storage>() as c::socklen_t;
@@ -270,7 +277,7 @@ pub(crate) fn acceptfrom(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketA
         ))?;
         Ok((
             owned_fd,
-            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+            maybe_read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
         ))
     }
 }
@@ -285,7 +292,7 @@ pub(crate) fn acceptfrom(sockfd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketA
 pub(crate) fn acceptfrom_with(
     sockfd: BorrowedFd<'_>,
     flags: AcceptFlags,
-) -> io::Result<(OwnedFd, SocketAddrAny)> {
+) -> io::Result<(OwnedFd, Option<SocketAddrAny>)> {
     unsafe {
         let mut storage = MaybeUninit::<c::sockaddr_storage>::uninit();
         let mut len = size_of::<c::sockaddr_storage>() as c::socklen_t;
@@ -297,7 +304,7 @@ pub(crate) fn acceptfrom_with(
         ))?;
         Ok((
             owned_fd,
-            read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
+            maybe_read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()),
         ))
     }
 }
@@ -315,7 +322,7 @@ pub(crate) fn accept_with(sockfd: BorrowedFd<'_>, _flags: AcceptFlags) -> io::Re
 pub(crate) fn acceptfrom_with(
     sockfd: BorrowedFd<'_>,
     _flags: AcceptFlags,
-) -> io::Result<(OwnedFd, SocketAddrAny)> {
+) -> io::Result<(OwnedFd, Option<SocketAddrAny>)> {
     acceptfrom(sockfd)
 }
 
@@ -339,7 +346,7 @@ pub(crate) fn getsockname(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
 }
 
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
-pub(crate) fn getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
+pub(crate) fn getpeername(sockfd: BorrowedFd<'_>) -> io::Result<Option<SocketAddrAny>> {
     unsafe {
         let mut storage = MaybeUninit::<c::sockaddr_storage>::uninit();
         let mut len = size_of::<c::sockaddr_storage>() as c::socklen_t;
@@ -348,7 +355,10 @@ pub(crate) fn getpeername(sockfd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
             storage.as_mut_ptr().cast(),
             &mut len,
         ))?;
-        Ok(read_sockaddr_os(storage.as_ptr(), len.try_into().unwrap()))
+        Ok(maybe_read_sockaddr_os(
+            storage.as_ptr(),
+            len.try_into().unwrap(),
+        ))
     }
 }
 
@@ -635,27 +645,27 @@ pub(crate) mod sockopt {
         #[cfg(not(any(
             target_os = "dragonfly",
             target_os = "freebsd",
-            target_os = "ios",
-            target_os = "macos",
-            target_os = "openbsd",
-            target_os = "netbsd",
-            target_os = "illumos",
-            target_os = "solaris",
             target_os = "haiku",
-            target_os = "l4re"
+            target_os = "illumos",
+            target_os = "ios",
+            target_os = "l4re",
+            target_os = "macos",
+            target_os = "netbsd",
+            target_os = "openbsd",
+            target_os = "solaris",
         )))]
         use c::IPV6_ADD_MEMBERSHIP;
         #[cfg(any(
             target_os = "dragonfly",
             target_os = "freebsd",
-            target_os = "ios",
-            target_os = "macos",
-            target_os = "openbsd",
-            target_os = "netbsd",
-            target_os = "illumos",
-            target_os = "solaris",
             target_os = "haiku",
-            target_os = "l4re"
+            target_os = "illumos",
+            target_os = "ios",
+            target_os = "l4re",
+            target_os = "macos",
+            target_os = "netbsd",
+            target_os = "openbsd",
+            target_os = "solaris",
         ))]
         use c::IPV6_JOIN_GROUP as IPV6_ADD_MEMBERSHIP;
 
@@ -682,27 +692,27 @@ pub(crate) mod sockopt {
         #[cfg(not(any(
             target_os = "dragonfly",
             target_os = "freebsd",
-            target_os = "ios",
-            target_os = "macos",
-            target_os = "openbsd",
-            target_os = "netbsd",
-            target_os = "illumos",
-            target_os = "solaris",
             target_os = "haiku",
-            target_os = "l4re"
+            target_os = "illumos",
+            target_os = "ios",
+            target_os = "l4re",
+            target_os = "macos",
+            target_os = "netbsd",
+            target_os = "openbsd",
+            target_os = "solaris",
         )))]
         use c::IPV6_DROP_MEMBERSHIP;
         #[cfg(any(
             target_os = "dragonfly",
             target_os = "freebsd",
-            target_os = "ios",
-            target_os = "macos",
-            target_os = "openbsd",
-            target_os = "netbsd",
-            target_os = "illumos",
-            target_os = "solaris",
             target_os = "haiku",
-            target_os = "l4re"
+            target_os = "illumos",
+            target_os = "ios",
+            target_os = "l4re",
+            target_os = "macos",
+            target_os = "netbsd",
+            target_os = "openbsd",
+            target_os = "solaris",
         ))]
         use c::IPV6_LEAVE_GROUP as IPV6_DROP_MEMBERSHIP;
 
