@@ -24,7 +24,7 @@ pub(in crate::imp) unsafe fn indirect_syscall0(
         "call {callee}",
         callee = in(reg) callee,
         inlateout("eax") nr.to_asm() => r0,
-        options(nostack, preserves_flags)
+        options(preserves_flags)
     );
     FromAsm::from_asm(r0)
 }
@@ -42,7 +42,7 @@ pub(in crate::imp) unsafe fn indirect_syscall1(
         callee = in(reg) callee,
         inlateout("eax") nr.to_asm() => r0,
         in("ebx") a0.to_asm(),
-        options(nostack, preserves_flags)
+        options(preserves_flags)
     );
     FromAsm::from_asm(r0)
 }
@@ -77,7 +77,7 @@ pub(in crate::imp) unsafe fn indirect_syscall2(
         inlateout("eax") nr.to_asm() => r0,
         in("ebx") a0.to_asm(),
         in("ecx") a1.to_asm(),
-        options(nostack, preserves_flags)
+        options(preserves_flags)
     );
     FromAsm::from_asm(r0)
 }
@@ -99,7 +99,7 @@ pub(in crate::imp) unsafe fn indirect_syscall3(
         in("ebx") a0.to_asm(),
         in("ecx") a1.to_asm(),
         in("edx") a2.to_asm(),
-        options(nostack, preserves_flags)
+        options(preserves_flags)
     );
     FromAsm::from_asm(r0)
 }
@@ -115,7 +115,7 @@ pub(in crate::imp) unsafe fn indirect_syscall4(
     a3: ArgReg<'_, A3>,
 ) -> RetReg<R0> {
     let r0;
-    // a3 should go in esi, but asm! won't let us use it as an operand.
+    // a3 should go in esi, but `asm!` won't let us use it as an operand.
     // temporarily swap it into place, and then swap it back afterward.
     //
     // Note that we hard-code the callee operand to use edi instead of
@@ -131,7 +131,7 @@ pub(in crate::imp) unsafe fn indirect_syscall4(
         in("ebx") a0.to_asm(),
         in("ecx") a1.to_asm(),
         in("edx") a2.to_asm(),
-        options(nostack, preserves_flags)
+        options(preserves_flags)
     );
     FromAsm::from_asm(r0)
 }
@@ -148,20 +148,18 @@ pub(in crate::imp) unsafe fn indirect_syscall5(
     a4: ArgReg<'_, A4>,
 ) -> RetReg<R0> {
     let r0;
-    // Oof. a3 should go in esi, and asm! won't let us use that register as
+    // Oof. a3 should go in esi, and `asm!` won't let us use that register as
     // an operand. And we can't request stack slots. And there are no other
     // registers free. Use eax as a temporary pointer to a slice, since it
     // gets clobbered as the return value anyway.
     asm!(
-        "push ebp",
         "push esi",
-        "push [eax + 0]",
-        "mov esi, [eax + 4]",
-        "mov eax, [eax + 8]",
-        "call [esp]",
+        "push DWORD PTR [eax + 0]",
+        "mov esi, DWORD PTR [eax + 4]",
+        "mov eax, DWORD PTR [eax + 8]",
+        "call DWORD PTR [esp]",
         "pop esi",
         "pop esi",
-        "pop ebp",
         inout("eax") &[callee as usize, a3.to_asm(), nr.to_asm()] => r0,
         in("ebx") a0.to_asm(),
         in("ecx") a1.to_asm(),
@@ -185,9 +183,9 @@ pub(in crate::imp) unsafe fn indirect_syscall6(
     a5: ArgReg<'_, A5>,
 ) -> RetReg<R0> {
     let r0;
-    // Oof again. a3 should go in esi, and a5 should go in ebp, and asm! won't
-    // let us use either of those registers as operands. And we can't request
-    // stack slots. And there are no other registers free. Use eax as a
+    // Oof again. a3 should go in esi, and a5 should go in ebp, and `asm!`
+    // won't let us use either of those registers as operands. And we can't
+    // request stack slots. And there are no other registers free. Use eax as a
     // temporary pointer to a slice, since it gets clobbered as the return
     // value anyway.
     //
@@ -196,11 +194,11 @@ pub(in crate::imp) unsafe fn indirect_syscall6(
     asm!(
         "push ebp",
         "push esi",
-        "push [eax + 0]",
-        "mov esi, [eax + 4]",
-        "mov ebp, [eax + 8]",
-        "mov eax, [eax + 12]",
-        "call [esp]",
+        "push DWORD PTR [eax + 0]",
+        "mov esi, DWORD PTR [eax + 4]",
+        "mov ebp, DWORD PTR [eax + 8]",
+        "mov eax, DWORD PTR [eax + 12]",
+        "call DWORD PTR [esp]",
         "pop esi",
         "pop esi",
         "pop ebp",
@@ -351,7 +349,7 @@ pub(in crate::imp) unsafe fn syscall4(
     a3: ArgReg<'_, A3>,
 ) -> RetReg<R0> {
     let r0;
-    // a3 should go in esi, but asm! won't let us use it as an operand.
+    // a3 should go in esi, but `asm!` won't let us use it as an operand.
     // Temporarily swap it into place, and then swap it back afterward.
     asm!(
         "xchg esi, {a3}",
@@ -402,8 +400,10 @@ pub(in crate::imp) unsafe fn syscall5(
     a4: ArgReg<'_, A4>,
 ) -> RetReg<R0> {
     let r0;
-    // As in syscall 4, use xchg to handle a3. a4 should go in edi, and
-    // we can use that register as an operand.
+    // As in `syscall4`, use xchg to handle a3. a4 should go in edi, and we can
+    // use that register as an operand. Unlike in `indirect_syscall5`, we don't
+    // have a `callee` operand taking up a register, so we have enough
+    // registers and don't need to use a slice.
     asm!(
         "xchg esi, {a3}",
         "int $$0x80",
@@ -430,6 +430,7 @@ pub(in crate::imp) unsafe fn syscall5_readonly(
     a4: ArgReg<'_, A4>,
 ) -> RetReg<R0> {
     let r0;
+    // See the comments in `syscall5`.
     asm!(
         "xchg esi, {a3}",
         "int $$0x80",
@@ -457,20 +458,13 @@ pub(in crate::imp) unsafe fn syscall6(
     a5: ArgReg<'_, A5>,
 ) -> RetReg<R0> {
     let r0;
-    // Oof. a3 should go in esi, and a5 should go in ebp, and asm! won't
-    // let us use either of those registers as operands. And we can't request
-    // stack slots. And there are no other registers free. Use eax as a
-    // temporary pointer to a slice, since it gets clobbered as the return
-    // value anyway.
-    //
-    // This is another reason that syscalls should be compiler intrinsics
-    // rather than inline asm.
+    // See the comments in `indirect_syscall6`.
     asm!(
         "push ebp",
         "push esi",
-        "mov esi, [eax + 0]",
-        "mov ebp, [eax + 4]",
-        "mov eax, [eax + 8]",
+        "mov esi, DWORD PTR [eax + 0]",
+        "mov ebp, DWORD PTR [eax + 4]",
+        "mov eax, DWORD PTR [eax + 8]",
         "int $$0x80",
         "pop esi",
         "pop ebp",
@@ -496,12 +490,13 @@ pub(in crate::imp) unsafe fn syscall6_readonly(
     a5: ArgReg<'_, A5>,
 ) -> RetReg<R0> {
     let r0;
+    // See the comments in `indirect_syscall6`.
     asm!(
         "push ebp",
         "push esi",
-        "mov esi, [eax + 0]",
-        "mov ebp, [eax + 4]",
-        "mov eax, [eax + 8]",
+        "mov esi, DWORD PTR [eax + 0]",
+        "mov ebp, DWORD PTR [eax + 4]",
+        "mov eax, DWORD PTR [eax + 8]",
         "int $$0x80",
         "pop esi",
         "pop ebp",
