@@ -6,7 +6,9 @@ mod write_sockaddr;
 
 pub(crate) mod ext;
 pub(crate) mod syscalls;
-pub(crate) use read_sockaddr::{read_sockaddr, read_sockaddr_os};
+pub(crate) use read_sockaddr::{
+    initialize_family_to_unspec, maybe_read_sockaddr_os, read_sockaddr, read_sockaddr_os,
+};
 pub(crate) use write_sockaddr::{encode_sockaddr_v4, encode_sockaddr_v6, write_sockaddr};
 
 pub use addr::{SocketAddrStorage, SocketAddrUnix};
