@@ -14,8 +14,9 @@ use super::super::conv::{
 };
 use super::super::reg::nr;
 use super::{
-    encode_sockaddr_v4, encode_sockaddr_v6, read_sockaddr_os, AcceptFlags, AddressFamily, Protocol,
-    RecvFlags, SendFlags, Shutdown, SocketFlags, SocketType,
+    encode_sockaddr_v4, encode_sockaddr_v6, initialize_family_to_unspec, maybe_read_sockaddr_os,
+    read_sockaddr_os, AcceptFlags, AddressFamily, Protocol, RecvFlags, SendFlags, Shutdown,
+    SocketFlags, SocketType,
 };
 use crate::fd::BorrowedFd;
 use crate::io::{self, OwnedFd};
@@ -23,10 +24,11 @@ use crate::net::{SocketAddrAny, SocketAddrUnix, SocketAddrV4, SocketAddrV6};
 use core::convert::TryInto;
 use core::mem::MaybeUninit;
 #[cfg(not(any(
+    target_arch = "aarch64",
+    target_arch = "mips64",
+    target_arch = "riscv64",
     target_arch = "x86",
     target_arch = "x86_64",
-    target_arch = "aarch64",
-    target_arch = "riscv64"
 )))]
 use linux_raw_sys::general::{__NR_recv, __NR_send};
 use linux_raw_sys::general::{sockaddr, sockaddr_in, sockaddr_in6, socklen_t};
@@ -209,7 +211,7 @@ pub(crate) fn accept_with(fd: BorrowedFd<'_>, flags: AcceptFlags) -> io::Result<
 }
 
 #[inline]
-pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddrAny)> {
+pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, Option<SocketAddrAny>)> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
         let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
@@ -222,7 +224,7 @@ pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddrA
         ))?;
         Ok((
             fd,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
+            maybe_read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
         ))
     }
     #[cfg(target_arch = "x86")]
@@ -240,7 +242,7 @@ pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddrA
         ))?;
         Ok((
             fd,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
+            maybe_read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
         ))
     }
 }
@@ -249,7 +251,7 @@ pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, SocketAddrA
 pub(crate) fn acceptfrom_with(
     fd: BorrowedFd<'_>,
     flags: AcceptFlags,
-) -> io::Result<(OwnedFd, SocketAddrAny)> {
+) -> io::Result<(OwnedFd, Option<SocketAddrAny>)> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
         let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
@@ -263,7 +265,7 @@ pub(crate) fn acceptfrom_with(
         ))?;
         Ok((
             fd,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
+            maybe_read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
         ))
     }
     #[cfg(target_arch = "x86")]
@@ -282,7 +284,7 @@ pub(crate) fn acceptfrom_with(
         ))?;
         Ok((
             fd,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
+            maybe_read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
         ))
     }
 }
@@ -312,10 +314,11 @@ pub(crate) fn send(fd: BorrowedFd<'_>, buf: &[u8], flags: SendFlags) -> io::Resu
     let (buf_addr, buf_len) = slice(buf);
 
     #[cfg(not(any(
+        target_arch = "aarch64",
+        target_arch = "mips64",
+        target_arch = "riscv64",
         target_arch = "x86",
         target_arch = "x86_64",
-        target_arch = "aarch64",
-        target_arch = "riscv64"
     )))]
     unsafe {
         ret_usize(syscall4_readonly(
@@ -327,9 +330,10 @@ pub(crate) fn send(fd: BorrowedFd<'_>, buf: &[u8], flags: SendFlags) -> io::Resu
         ))
     }
     #[cfg(any(
-        target_arch = "x86_64",
         target_arch = "aarch64",
-        target_arch = "riscv64"
+        target_arch = "mips64",
+        target_arch = "riscv64",
+        target_arch = "x86_64",
     ))]
     unsafe {
         ret_usize(syscall6_readonly(
@@ -476,10 +480,11 @@ pub(crate) fn recv(fd: BorrowedFd<'_>, buf: &mut [u8], flags: RecvFlags) -> io::
     let (buf_addr_mut, buf_len) = slice_mut(buf);
 
     #[cfg(not(any(
+        target_arch = "aarch64",
+        target_arch = "mips64",
+        target_arch = "riscv64",
         target_arch = "x86",
         target_arch = "x86_64",
-        target_arch = "aarch64",
-        target_arch = "riscv64"
     )))]
     unsafe {
         ret_usize(syscall4(
@@ -491,9 +496,10 @@ pub(crate) fn recv(fd: BorrowedFd<'_>, buf: &mut [u8], flags: RecvFlags) -> io::
         ))
     }
     #[cfg(any(
-        target_arch = "x86_64",
         target_arch = "aarch64",
-        target_arch = "riscv64"
+        target_arch = "mips64",
+        target_arch = "riscv64",
+        target_arch = "x86_64",
     ))]
     unsafe {
         ret_usize(syscall6(
@@ -526,13 +532,19 @@ pub(crate) fn recvfrom(
     fd: BorrowedFd<'_>,
     buf: &mut [u8],
     flags: RecvFlags,
-) -> io::Result<(usize, SocketAddrAny)> {
+) -> io::Result<(usize, Option<SocketAddrAny>)> {
     let (buf_addr_mut, buf_len) = slice_mut(buf);
 
-    #[cfg(not(target_arch = "x86"))]
+    let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
+    let mut storage = MaybeUninit::<sockaddr>::uninit();
+
     unsafe {
-        let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
-        let mut storage = MaybeUninit::<sockaddr>::uninit();
+        // `recvfrom` does not write to the storage if the socket is
+        // connection-oriented sockets, so we initialize the family field to
+        // `AF_UNSPEC` so that we can detect this case.
+        initialize_family_to_unspec(storage.as_mut_ptr());
+
+        #[cfg(not(target_arch = "x86"))]
         let nread = ret_usize(syscall6(
             nr(__NR_recvfrom),
             borrowed_fd(fd),
@@ -542,15 +554,7 @@ pub(crate) fn recvfrom(
             out(&mut storage),
             by_mut(&mut addrlen),
         ))?;
-        Ok((
-            nread,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
-        ))
-    }
-    #[cfg(target_arch = "x86")]
-    unsafe {
-        let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
-        let mut storage = MaybeUninit::<sockaddr>::uninit();
+        #[cfg(target_arch = "x86")]
         let nread = ret_usize(syscall2(
             nr(__NR_socketcall),
             x86_sys(SYS_RECVFROM),
@@ -563,15 +567,16 @@ pub(crate) fn recvfrom(
                 by_mut(&mut addrlen),
             ]),
         ))?;
+
         Ok((
             nread,
-            read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
+            maybe_read_sockaddr_os(&storage.assume_init(), addrlen.try_into().unwrap()),
         ))
     }
 }
 
 #[inline]
-pub(crate) fn getpeername(fd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
+pub(crate) fn getpeername(fd: BorrowedFd<'_>) -> io::Result<Option<SocketAddrAny>> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
         let mut addrlen = core::mem::size_of::<sockaddr>() as socklen_t;
@@ -582,7 +587,7 @@ pub(crate) fn getpeername(fd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
             out(&mut storage),
             by_mut(&mut addrlen),
         ))?;
-        Ok(read_sockaddr_os(
+        Ok(maybe_read_sockaddr_os(
             &storage.assume_init(),
             addrlen.try_into().unwrap(),
         ))
@@ -600,7 +605,7 @@ pub(crate) fn getpeername(fd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
                 by_mut(&mut addrlen),
             ]),
         ))?;
-        Ok(read_sockaddr_os(
+        Ok(maybe_read_sockaddr_os(
             &storage.assume_init(),
             addrlen.try_into().unwrap(),
         ))
