@@ -27,9 +27,19 @@ use crate::process::{
 };
 use core::convert::TryInto;
 use core::mem::MaybeUninit;
-#[cfg(not(any(target_arch = "arm", target_arch = "powerpc64", target_arch = "x86")))]
+#[cfg(not(any(
+    target_arch = "arm",
+    target_arch = "powerpc",
+    target_arch = "powerpc64",
+    target_arch = "x86"
+)))]
 use linux_raw_sys::general::__NR_getrlimit;
-#[cfg(any(target_arch = "arm", target_arch = "powerpc64", target_arch = "x86"))]
+#[cfg(any(
+    target_arch = "arm",
+    target_arch = "powerpc",
+    target_arch = "powerpc64",
+    target_arch = "x86"
+))]
 use linux_raw_sys::general::__NR_ugetrlimit as __NR_getrlimit;
 use linux_raw_sys::general::{
     __NR_chdir, __NR_exit_group, __NR_fchdir, __NR_getcwd, __NR_getpid, __NR_getppid,
@@ -58,6 +68,7 @@ pub(crate) fn getcwd(buf: &mut [u8]) -> io::Result<usize> {
     unsafe { ret_usize(syscall2(nr(__NR_getcwd), buf_addr_mut, buf_len)) }
 }
 
+#[inline]
 pub(crate) fn membarrier_query() -> MembarrierQuery {
     unsafe {
         match ret_c_uint(syscall2(
@@ -79,6 +90,7 @@ pub(crate) fn membarrier_query() -> MembarrierQuery {
     }
 }
 
+#[inline]
 pub(crate) fn membarrier(cmd: MembarrierCommand) -> io::Result<()> {
     unsafe {
         ret(syscall2(
@@ -89,6 +101,7 @@ pub(crate) fn membarrier(cmd: MembarrierCommand) -> io::Result<()> {
     }
 }
 
+#[inline]
 pub(crate) fn membarrier_cpu(cmd: MembarrierCommand, cpu: Cpuid) -> io::Result<()> {
     unsafe {
         ret(syscall3(
