@@ -27,9 +27,8 @@ pub use imp::io::DupFlags;
 /// [Linux]: https://man7.org/linux/man-pages/man2/dup.2.html
 #[cfg(not(target_os = "wasi"))]
 #[inline]
-pub fn dup<Fd: AsFd>(fd: &Fd) -> io::Result<OwnedFd> {
-    let fd = fd.as_fd();
-    imp::io::syscalls::dup(fd)
+pub fn dup<Fd: AsFd>(fd: Fd) -> io::Result<OwnedFd> {
+    imp::io::syscalls::dup(fd.as_fd())
 }
 
 /// `dup2(fd, new)`—Creates a new `OwnedFd` instance that shares the
@@ -49,9 +48,8 @@ pub fn dup<Fd: AsFd>(fd: &Fd) -> io::Result<OwnedFd> {
 /// [Linux]: https://man7.org/linux/man-pages/man2/dup2.2.html
 #[cfg(not(target_os = "wasi"))]
 #[inline]
-pub fn dup2<Fd: AsFd>(fd: &Fd, new: &OwnedFd) -> io::Result<()> {
-    let fd = fd.as_fd();
-    imp::io::syscalls::dup2(fd, new)
+pub fn dup2<Fd: AsFd>(fd: Fd, new: &OwnedFd) -> io::Result<()> {
+    imp::io::syscalls::dup2(fd.as_fd(), new)
 }
 
 /// `dup3(fd, new, flags)`—Creates a new `OwnedFd` instance that shares the
@@ -70,7 +68,6 @@ pub fn dup2<Fd: AsFd>(fd: &Fd, new: &OwnedFd) -> io::Result<()> {
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[doc(alias = "dup3")]
-pub fn dup2_with<Fd: AsFd>(fd: &Fd, new: &OwnedFd, flags: DupFlags) -> io::Result<()> {
-    let fd = fd.as_fd();
-    imp::io::syscalls::dup2_with(fd, new, flags)
+pub fn dup2_with<Fd: AsFd>(fd: Fd, new: &OwnedFd, flags: DupFlags) -> io::Result<()> {
+    imp::io::syscalls::dup2_with(fd.as_fd(), new, flags)
 }
