@@ -2,7 +2,16 @@
 //! pointers.
 #![cfg_attr(target_vendor = "mustang", allow(unsafe_code))]
 
-#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
+#[cfg(any(
+    linux_raw,
+    all(
+        libc,
+        any(
+            all(target_os = "android", target_pointer_width = "64"),
+            target_os = "linux"
+        )
+    )
+))]
 use crate::ffi::ZStr;
 use crate::imp;
 
@@ -17,13 +26,22 @@ pub fn page_size() -> usize {
 /// data.
 ///
 /// Return the Linux `AT_HWCAP` and `AT_HWCAP2` values passed to the
-/// current process.
+/// current process. Returns 0 for each value if it is not available.
 ///
 /// # References
 ///  - [Linux]
 ///
 /// [Linux]: https://man7.org/linux/man-pages/man3/getauxval.3.html
-#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
+#[cfg(any(
+    linux_raw,
+    all(
+        libc,
+        any(
+            all(target_os = "android", target_pointer_width = "64"),
+            target_os = "linux"
+        )
+    )
+))]
 #[inline]
 pub fn linux_hwcap() -> (usize, usize) {
     imp::process::linux_hwcap()
@@ -32,13 +50,22 @@ pub fn linux_hwcap() -> (usize, usize) {
 /// `getauxval(AT_EXECFN)`—Returns the Linux "execfn" string.
 ///
 /// Return the string that Linux has recorded as the filesystem path to the
-/// executable.
+/// executable. Returns an empty string if the string is not available.
 ///
 /// # References
 ///  - [Linux]
 ///
 /// [Linux]: https://man7.org/linux/man-pages/man3/getauxval.3.html
-#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
+#[cfg(any(
+    linux_raw,
+    all(
+        libc,
+        any(
+            all(target_os = "android", target_pointer_width = "64"),
+            target_os = "linux"
+        )
+    )
+))]
 #[inline]
 pub fn linux_execfn() -> &'static ZStr {
     imp::process::linux_execfn()
